package kaysaar.aotd_question_of_loyalty.data.models;

import kaysaar.aotd_question_of_loyalty.data.tags.AoTDRankTags;
import java.util.Iterator;
import java.util.HashSet;
import kaysaar.aotd_question_of_loyalty.data.scripts.commision.AoTDCommissionDataManager;
import com.fs.starfarer.api.campaign.FactionAPI;
import com.fs.starfarer.api.EveryFrameScript;
import com.fs.starfarer.api.Global;
import kaysaar.aotd_question_of_loyalty.data.intel.AoTDCommIntelPlugin;
import java.util.Set;
import java.util.HashMap;
import java.util.ArrayList;
import java.util.LinkedHashMap;

public class BaseFactionCommisionData
{
    public String factionID;
    public LinkedHashMap<String, String> iconMap;
    public ArrayList<RankData> ranks;
    public HashMap<String, Integer> rankValue;
    public int maxProgress;
    public String intelPlugin;
    public Set<String> tags;
    
    public AoTDCommIntelPlugin getPlugin() {
        try {
            final AoTDCommIntelPlugin plugin = (AoTDCommIntelPlugin)Global.getSettings().getScriptClassLoader().loadClass(this.intelPlugin).newInstance();
            plugin.setData(this);
            Global.getSector().getListenerManager().removeListener((Object)plugin);
            Global.getSector().removeScript((EveryFrameScript)plugin);
            return plugin;
        }
        catch (final Exception e) {
            throw new RuntimeException(e);
        }
    }
    
    public FactionAPI getFaction() {
        return Global.getSector().getFaction(this.factionID);
    }
    
    public BaseFactionCommisionData(final String factionID, final LinkedHashMap<String, String> iconMap, final int maxPoints, final ArrayList<RankData> ranks, final String plugin, final HashMap<String, Integer> rankValue, final Set tags) {
        this.factionID = factionID;
        this.iconMap = iconMap;
        this.maxProgress = maxPoints;
        this.ranks = ranks;
        this.intelPlugin = plugin;
        this.rankValue = rankValue;
        this.tags = tags;
    }
    
    public static BaseFactionCommisionData getDefaultCommisionData(final String factionID) {
        final LinkedHashMap<String, String> iconMap = new LinkedHashMap<String, String>();
        final ArrayList<RankData> data = new ArrayList<RankData>();
        data.add(AoTDCommissionDataManager.getInstance().getRank("mercenary"));
        data.add(AoTDCommissionDataManager.getInstance().getRank("auxiliary"));
        data.add(AoTDCommissionDataManager.getInstance().getRank("commander"));
        data.add(AoTDCommissionDataManager.getInstance().getRank("admiral"));
        data.add(AoTDCommissionDataManager.getInstance().getRank("planetary_governor"));
        data.add(AoTDCommissionDataManager.getInstance().getRank("system_governor"));
        data.add(AoTDCommissionDataManager.getInstance().getRank("grand_moff"));
        final HashMap<String, Integer> rankValue = new HashMap<String, Integer>();
        rankValue.put("mercenary", 0);
        rankValue.put("auxiliary", 250);
        rankValue.put("commander", 500);
        rankValue.put("admiral", 750);
        rankValue.put("planetary_governor", 1200);
        rankValue.put("system_governor", 1700);
        rankValue.put("grand_moff", 2200);
        final HashSet<String> tags = new HashSet<String>();
        iconMap.put("grand_moff", "hyperspace_topography");
        iconMap.put("system_governor", "hyperspace_topography");
        iconMap.put("planetary_governor", "hyperspace_topography");
        iconMap.put("admiral", "hyperspace_topography");
        iconMap.put("commander", "hyperspace_topography");
        iconMap.put("auxiliary", "hyperspace_topography");
        iconMap.put("mercenary", "hyperspace_topography");
        return new BaseFactionCommisionData(factionID, iconMap, 2500, data, AoTDCommIntelPlugin.class.getName(), rankValue, tags);
    }
    
    public String getFirstDefRank() {
        for (final RankData rank : this.ranks) {
            if (rank.isLeavingAnOption) {
                return rank.id;
            }
        }
        return null;
    }
    
    public String getFirstOfficialRank() {
        for (final RankData rank : this.ranks) {
            if (!rank.isLeavingAnOption) {
                return rank.id;
            }
        }
        return null;
    }
    
    public RankData getAdequateRankForColonies(final int colonyNumber) {
        RankData firstNonRestrictiveRank = null;
        for (final RankData rank : this.ranks) {
            if (rank.hasTag(AoTDRankTags.NONRESTRICTIVE_COLONIZATION)) {
                firstNonRestrictiveRank = rank;
            }
            if (rank.getAmountOfColoniesAbleToColonize() >= colonyNumber) {
                return rank;
            }
        }
        if (firstNonRestrictiveRank == null) {
            firstNonRestrictiveRank = this.getRankWithMaxColonies();
        }
        return firstNonRestrictiveRank;
    }
    
    public RankData getRankWithMaxColonies() {
        RankData firstNonRestrictiveRank = this.ranks.get(0);
        for (final RankData rank : this.ranks) {
            if (rank.getAmountOfColoniesAbleToColonize() != 0 && firstNonRestrictiveRank == null) {
                firstNonRestrictiveRank = rank;
            }
            else {
                if (rank.getAmountOfColoniesAbleToColonize() <= firstNonRestrictiveRank.getAmountOfColoniesAbleToColonize()) {
                    continue;
                }
                firstNonRestrictiveRank = rank;
            }
        }
        return firstNonRestrictiveRank;
    }
    
    public RankData getRankFromString(final String rank) {
        for (final RankData rankData : this.ranks) {
            if (rankData.id.equals(rank)) {
                return rankData;
            }
        }
        return null;
    }
    
    public boolean hasTag(final String tag) {
        for (final String s : this.tags) {
            if (s.equals(tag)) {
                return true;
            }
        }
        return false;
    }
}
